#ifndef EVENT_QUEUE_H
#define EVENT_QUEUE_H

#include <stdio.h>
#include <stdlib.h>
#include "event.h"

#define QUEUE_TIMEOUT 1
#define NAMED_PIPE_PATH "/tmp/np_event"
#define UDP_SOCKET_PORT 33000


enum event_queue_state { HALT = 0, RUNNING = 1 };

typedef enum event_queue_state eventqueue_state;

/* struct event_queue
 * 
 * Enthält Informationen zum Eventhandler:
 * 	* Function
 *  * params
 *  * id
 * 
 * Die EventQueue enthält mehrere solcher Strukturen. Die Events(-handler)
 * sollen anhand der ID auch wieder entfernt werden können.
 * 
 */

typedef struct event_queue {
  int next_id;

  eventqueue_state state;
} EventQueue;

/* new_event_queue()
 * 
 * erzeugt eine neue event_queue. Die Queue enthält keine Event(s|-handler).
 * Events die eingefügt werden, werden noch nicht bearbeitet, d.h. an dieser stelle werden
 * Socket-, Pipe-,...-events ignoriert.
 */

EventQueue *new_event_queue();

/* insert_event
 * 
 * Fügt neue Event(handler) hinzu.
 * 
 * Parameter:
 * - eq: eine EventQueue (siehe new_event_queue)
 * - type: Art des Events (siehe eventtype in event.h)
 * - prio: Priorität (wird ignoriert)
 * - int32_t (*func)(void*,void*): Pointer auf eine Funktion (z.B. int32_t foo(void *p,void *d))
 *                                 der erste Parameter ist hierbei param
 *                                 der zweite Parameter sind die Daten des Events
 * 
 * - param: erste parameter der der function "func" übergeben wird
 * 
 * Rückgabe: event-ID ! mit hilfe dieser ID kann der Handler wieder entfernt werden
 */
int insert_event(EventQueue *eq, eventtype type, int prio, int32_t (*func)(void*,void*), void *param);

/* insert_time_event
 * 
 * ähnlich wie insert_event
 * jedoch:
 * 1. kein type (der is klar)
 * 2. delay: -zeit bis das event ausgelöst wird
 * 
 * Ausgelöst wird das Event zum Zeitpunkt T = "Zeit_der_eventqueue_bei_Aufruf" + delay
 * Wenn die Zeit T erreichtist (Zeit der Eventqueue == T) wird das Event ausgelöst
 * 
 */
  
int insert_time_event(EventQueue *eq, int prio, int32_t (*func)(void*,void*), void *param, int delay);

/* remove_event
 * 
 * entfernt ein Event(handler) mit der ID "id" (siehe insert_(time_)event )
 * 
 */
int remove_event(EventQueue *eq, int id);

/* run_event_queue
 * 
 * startet die Eventqueue. von da an werden die event bearbeitet. Der Aufruf kehrt erst zurück, wenn 
 * "stop_event_queue(EventQueue *eq)" aufgerufen wird
 *
 * Ab diesem Aufruf läuft auch die Uhr der Eventqueue (time_events).
 * Die Uhr läuft im Sekundentakt und startet bei 0 (Aufruf von run_event_queue)
 * 
 */
void run_event_queue(EventQueue *eq);

/* stop_event_queue
 * 
 * stoppt die Eventqueue. Die Evnts werden nicht mehr bearbeitet. Der run_event_queue - Aufruf kehrt zurück.
 * 
 */
void stop_event_queue(EventQueue *eq);

/* delete_event_queue
 * 
 * löscht die Eventqueue. Offene Ports etc. müssen spätestens hier geschlossen werden.
 * 
 */
void delete_event_queue(EventQueue *eq);

/* get_time
 * 
 * Gibt die aktuelle Zeit der Eventqueue zurück. Die Uhr läuft im Sekundentakt.
 */
int get_time(EventQueue *eq);

#endif
